/// <reference path="../native.ts" />
/// <reference path="../object.ts" />

namespace trusted.cms {
    /**
     * Wrap CMS_SignerInfo
     *
     * @export
     * @class Signer
     * @extends {BaseObject<native.CMS.Signer>}
     */
    export class Signer extends BaseObject<native.CMS.Signer> {

        /**
         * Creates an instance of Signer.
         *
         * @param {native.CMS.Signer} handle
         *
         * @memberOf Signer
         */
        constructor(nativeHandle?: native.CMS.Signer) {
            super();

            if (nativeHandle instanceof native.CMS.Signer) {
                this.handle = nativeHandle;
            }
            else {
                this.handle = new native.CMS.Signer();
            }
        }

        /**
         * Return signer certificate
         *
         * @type {Certificate}
         * @memberOf Signer
         */
        get certificate(): pki.Certificate {
            const cert = this.handle.getCertificate();

            if (cert) {
                return pki.Certificate.wrap<native.PKI.Certificate, pki.Certificate>(cert);
            } else {
                return undefined;
            }
        }

        /**
         * Set signer certificate
         * Error if cert no signer
         *
         * @param cert Certificate
         *
         * @memberOf Signer
         */
        set certificate(cert: pki.Certificate) {
            this.handle.setCertificate(cert.handle);
        }

        /**
         * Return Index
         *
         * @readonly
         * @type {number}
         * @memberOf Signer
         */
        get index(): number {
            return this.handle.getIndex();
        }

        /**
         * Set index certificate
         *
         * @param ind string
         *
         * @memberOf Signer
         */
        set index(ind: number) {
            this.handle.setIndex(ind);
        }

        /**
         * Return signing time from signed attributes
         *
         * @readonly
         * @type {Date}
         * @memberof Signer
         */
        get signingTime(): Date {
            const strDate = this.handle.getSigningTime();

            if (!strDate.length) {
                return undefined;
            }

            return new Date(strDate);
        }

        /**
        * Return signature algorithm
        *
        * @readonly
        * @type {string}
        * @memberOf Signer
        */
        get signatureAlgorithm(): string {
            return this.handle.getSignatureAlgorithm();
        }

        /**
         * Return signature digest algorithm
         *
         * @readonly
         * @type {string}
         * @memberOf Signer
         */
        get signatureDigestAlgorithm(): string {
            return this.handle.getDigestAlgorithm();
        }

        /**
         * Return issuer name
         *
         * @readonly
         * @type {string}
         * @memberOf Signer
         */
        get issuerName(): string {
            return this.handle.getIssuerName();
        }

        /**
         * Return serial number of certificate
         *
         * @readonly
         * @type {string}
         * @memberOf Signer
         */
        get serialNumber(): string {
            return this.handle.getSerialNumber();
        }

        /**
         * Return time stamp of specified type
         *
         * @type {TSP}
         * @memberOf Signer
         */
        public timestamp(tspType: number): pki.TSP {
            if (typeof tspType !== "number")
                throw new TypeError("Signer::signParams: Wrong input param");

            var resultTsp = undefined;

            var timeStamp = this.handle.timestamp(tspType);
            if (timeStamp !== undefined)
                resultTsp = new pki.TSP(timeStamp);

            return resultTsp;
        }

        /**
         * Verifyes time stamp of specified type from signer
         *
         * @type {boolean}
         * @memberOf Signer
         */
        public verifyTimestamp(tspType: number): boolean {
            if (typeof tspType !== "number")
                throw new TypeError("Signer::signParams: Wrong input param");

            return this.handle.verifyTimestamp(tspType);
        }

        /**
         * Identify if signer is CAdES or not
         *
         * @readonly
         * @type {boolean}
         * @memberOf Signer
         */
        get isCades(): boolean {
            return this.handle.isCades();
        }

        /**
         * Return CAdES type
         *
         * @type {CadesType}
         * @memberOf CadesParams
         */
        get cadesType(): CadesType {
            return this.handle.cadesType();
        }

        /**
         * For CAdES returns collection of certificates from certificateValues attribute
         *
         * @type {CertificateCollection}
         * @memberOf Signer
         */
        get certificateValues(): pki.CertificateCollection {
            const certVals = this.handle.certificateValues();

            if (certVals === undefined) {
                return undefined;
            }

            return new pki.CertificateCollection(certVals);
        }

        /**
         * For CAdES returns array of buffers with encoded revocation values (OCSP response or CRL)
         *
         * @readonly
         * @type {Buffer[]}
         * @memberOf Signer
         */
        get revocationValues(): Buffer[] {
            return this.handle.revocationValues();
        }

        /**
         * For CAdES returns OCSP response
         *
         * @readonly
         * @type {OCSP}
         * @memberOf Signer
         */
        get ocspResp(): pki.OCSP {
            var resultOcsp = undefined;

            var resp = this.handle.ocspResp();
            if (resp !== undefined)
                resultOcsp = new pki.OCSP(resp);

            return resultOcsp;
        }

        /**
         * Verify signature by hash and value
         *
         * @static
         * @param {Buffer} [hashValue] Buffer with hash value
         * @param {Buffer} [signValue] Buffer with signature value
         * @param {string | undefined} [signAlg] Signature algorithm OID (combined with hash alg)
         * @param {pki.Certificate | Buffer} [signCertOrKeyValue] Signature certificate or Buffer with key value (encoded SubjectPublicKeyInfo)
         * @returns {boolean} Signature verification result
         * @memberof Csp
         */
        public static verifySignatureValue(
            hashValue: Buffer,
            signValue: Buffer,
            signAlg: string | undefined,
            signCertOrKeyValue: pki.Certificate | Buffer,
            keyAlg?: string,
            namedCurve?: string,
        ): boolean {
            const signer = new native.CMS.Signer();
            if (signCertOrKeyValue instanceof pki.Certificate) {
                return signer.verifySignatureValue(hashValue, signValue, signAlg, signCertOrKeyValue.handle);
            } else if (signCertOrKeyValue instanceof Buffer) {
                return signer.verifySignatureValue(hashValue, signValue, signAlg, signCertOrKeyValue, keyAlg, namedCurve);
            }

            throw new TypeError("Signer::verifySignatureValue: Wrong input param");
        }
    }
}
