/// <reference path="../native.ts" />
/// <reference path="../object.ts" />

namespace trusted.pki {

    const DEFAULT_DATA_FORMAT: DataFormat = DataFormat.DER;

    /**
     * Wrap CRL
     *
     * @export
     * @class CRL
     * @extends {BaseObject<native.PKI.CRL>}
     */
    export class CRL extends BaseObject<native.PKI.CRL> {
        /**
         * Load CRL from file
         *
         * @static
         * @param {string} filename File location
         * @param {DataFormat} [format] PEM | DER
         * @returns {CRL}
         *
         * @memberOf CRL
         */
        public static load(filename: string, format?: DataFormat): CRL {
            const crl: CRL = new CRL();
            crl.handle.load(filename, format);
            return crl;
        }

        /**
         * Load CRL from memory
         *
         * @static
         * @param {Buffer} buffer
         * @param {DataFormat} [format=DEFAULT_DATA_FORMAT] PEM | DER (default)
         * @returns {CRL}
         *
         * @memberOf CRL
         */
        public static import(buffer: Buffer, format: DataFormat = DEFAULT_DATA_FORMAT): CRL {
            const crl: CRL = new CRL();
            crl.handle.import(buffer, format);
            return crl;
        }

        /**
         * Creates an instance of CRL.
         * @param {native.PKI.CRL} [param]
         *
         * @memberOf Certificate
         */
        constructor(param?: native.PKI.CRL) {
            super();
            if (param instanceof native.PKI.CRL) {
                this.handle = param;
            } else {
                this.handle = new native.PKI.CRL();
            }
        }

        /**
         * Return version of CRL
         *
         * @readonly
         * @type {number}
         * @memberOf Certificate
         */
        get version(): number {
            return this.handle.getVersion();
        }

         /**
         * Return issuer name
         *
         * @readonly
         * @type {string}
         * @memberOf CRL
         */
        get issuerName(): string {
            return this.handle.getIssuerName();
        }

        /**
         * Return CN from issuer name
         *
         * @readonly
         * @type {string}
         * @memberOf CRL
         */
        get issuerFriendlyName(): string {
            return this.handle.getIssuerFriendlyName();
        }

        /**
         * Return last update date
         *
         * @readonly
         * @type {Date}
         * @memberOf CRL
         */
        get lastUpdate(): Date {
            return new Date(this.handle.getLastUpdate());
        }

        /**
         * Return next update date
         *
         * @readonly
         * @type {Date}
         * @memberOf CRL
         */
        get nextUpdate(): Date {
            return new Date(this.handle.getNextUpdate());
        }

        /**
         * Return SHA-1 thumbprint
         *
         * @readonly
         * @type {string}
         * @memberOf CRL
         */
        get thumbprint(): string {
            return this.handle.getThumbprint().toString("hex");
        }

        /**
         * Return signature algorithm
         *
         * @readonly
         * @type {string}
         * @memberOf CRL
         */
        get signatureAlgorithm(): string {
            return this.handle.getSignatureAlgorithm();
        }

        /**
         * Return signature digest algorithm
         *
         * @readonly
         * @type {string}
         * @memberOf CRL
         */
        get signatureDigestAlgorithm(): string {
            return this.handle.getSignatureDigestAlgorithm();
        }

        /**
         * Return authority keyid
         *
         * @readonly
         * @type {string}
         * @memberOf CRL
         */
        get authorityKeyid(): string {
            return this.handle.getAuthorityKeyid().toString("hex");
        }

        /**
         * Return CRL number
         *
         * @readonly
         * @type {number}
         * @memberOf CRL
         */
        get crlNumber(): number {
            return this.handle.getCrlNumber();
        }

        /**
         * Load CRL from file
         *
         * @param {string} filename File location
         * @param {DataFormat} [format=DEFAULT_DATA_FORMAT] PEM | DER (default)
         *
         * @memberOf CRL
         */
        public load(filename: string, format?: DataFormat): void {
            this.handle.load(filename, format);
        }

        /**
         * Load CRL from memory
         *
         * @param {Buffer} buffer
         * @param {DataFormat} [format=DEFAULT_DATA_FORMAT]
         *
         * @memberOf CRL
         */
        public import(buffer: Buffer, format: DataFormat = DEFAULT_DATA_FORMAT): void {
            this.handle.import(buffer, format);
        }

        /**
         * Save CRL to memory
         *
         * @param {DataFormat} [format=DEFAULT_DATA_FORMAT]
         * @returns {Buffer}
         *
         * @memberOf CRL
         */
        public export(format: DataFormat = DEFAULT_DATA_FORMAT): Buffer {
            return this.handle.export(format);
        }

        /**
         * Write CRL to file
         *
         * @param {string} filename File location
         * @param {DataFormat} [dataFormat=DEFAULT_DATA_FORMAT]
         *
         * @memberOf CRL
         */
        public save(filename: string, dataFormat: DataFormat = DEFAULT_DATA_FORMAT): void {
            this.handle.save(filename, dataFormat);
        }

        /**
         * Compare CRLs
         *
         * @param {CRL} crl CRL for compare
         * @returns {number}
         *
         * @memberOf CRL
         */
        public compare(crl: CRL): number {
            const cmp: number = this.handle.compare(crl.handle);
            if (cmp < 0) {
                return -1;
            }
            if (cmp > 0) {
                return 1;
            }

            return 0;
        }

        /**
         * Compare CRLs
         *
         * @param {CRL} crl CRL for compare
         * @returns {boolean}
         *
         * @memberOf CRL
         */
        public equals(crl: CRL): boolean {
            return this.handle.equals(crl.handle);
        }

        /**
         * Return CRL hash
         *
         * @param {string} [algorithm="sha1"]
         * @returns {String}
         *
         * @memberOf CRL
         */
        public hash(algorithm: string = "sha1"): string {
            return this.handle.hash(algorithm).toString("hex");
        }

        /**
         * Return CRL duplicat
         *
         * @returns {CRL}
         *
         * @memberOf CRL
         */
        public duplicate(): CRL {
            const crl: CRL = new CRL();
            crl.handle = this.handle.duplicate();
            return crl;
        }
    }
}
