/// <reference path="../native.ts" />
/// <reference path="../object.ts" />

namespace trusted.pki {

    // Enums values copy from OCSP SDK
    export enum CPRespStatus {
        /// запрос обработан успешно
        successful = 0,
        /// неправильный запрос
        malformedRequest = 1,
        /// запрос не обработан из-за внутренней ошибки
        internalError = 2,
        /// сервис временно не может ответить, попробуйте позже
        tryLater = 3,
        /// запрос должен быть подписан
        sigRequired = 5,
        /// отправитель запроса не уполномочен осуществлять такой запрос
        unauthorized = 6,
        /// ошибка при обработке CRL
        badCRL = 8
    }

    export enum CPCertStatus {
        /// сертификат не отозван
        Good = 0,
        /// сертификат отозван
        Revoked = 1,
        /// статус сертификата неизвестен
        Unknown = 2
    }
    export enum CPCrlReason {
        /// не определена причина отзыва
        CRLREASON_UNSPECIFIED = 0,
        /// скопроментирован сертификат
        CRLREASON_KEYCOMPROMISE = 1,
        /// скомпроментирован центр сертификации
        CRLREASON_CACOMPROMISE = 2,
        /// изменилась нформация в сертификате
        CRLREASON_AFFILIATIONCHANGED = 3,
        /// сертификат замещен
        CRLREASON_SUPERSEDED = 4,
        /// сертификат больше не нужен для тех целей для которых выпускался
        CRLREASON_CESSATIONOFOPERATION = 5,
        /// действие сертификата приостановлено
        CRLREASON_CERTIFICATEHOLD = 6,
        /// убрать серификат из CRL (используется только в разностных CRL)
        CRLREASON_REMOVEFROMCRL = 8,
        /// привилегия, подтверждаемая данным сертификатом была отозвана
        CRLREASON_PRIVILEDGEWITHDRAWN = 9,
        /// скомпроментирован центр, выпускающий атрибутные сертификаты
        CRLREASON_AACOMPROMISE = 10
    }

    /**
     * Wrap OCSP Response and request sending
     *
     * @export
     * @class OCSP
     * @extends {BaseObject<native.PKI.OCSP>}
     */
    export class OCSP extends BaseObject<native.PKI.OCSP> {
        /**
         * Creates an instance of Ocsp.
         * @param {native.PKI.Certificate | Buffer, native.UTILS.ConnectionSettings?} [param]
         *
         * @memberOf Certificate
         */
        constructor(inData: Certificate | Buffer | native.PKI.OCSP, connSettings?: trusted.utils.ConnectionSettings) {
            super();
            if (inData instanceof Certificate) {
                this.handle = new native.PKI.OCSP(inData.handle, (connSettings != undefined) ? connSettings.handle : new native.UTILS.ConnectionSettings());
            } else if (inData instanceof Buffer) {
                this.handle = new native.PKI.OCSP(inData);
            } else if (inData instanceof native.PKI.OCSP) {
                this.handle = inData;
            } else {
                throw new TypeError("OCSP::constructor: Wrong input param");
            }
        }

        public Export(): Buffer {
            return this.handle.Export();
        }

        /**
         * Verify response signature with specified certificate. If certificate not cpecified, internal certificates used.
         * On success returns 0 and on error returns error code.
         *
         * @param {Certificate} serviceCert
         * @returns {number}
         *
         * @memberOf OCSP
         */
        public Verify(serviceCert?: Certificate): number {
        if (serviceCert instanceof Certificate)
            return this.handle.Verify(serviceCert.handle);

            return this.handle.Verify();
        }

        public VerifyCertificate(cert: Certificate): number {
            return this.handle.VerifyCertificate( cert.handle );
        }

        public get RespStatus(): CPRespStatus {
            return this.handle.RespStatus();
        }

        public get SignatureAlgorithmOid(): string {
            return this.handle.SignatureAlgorithmOid();
        }

        public get Certificates(): CertificateCollection {
            return CertificateCollection.wrap<native.PKI.CertificateCollection, CertificateCollection>(this.handle.Certificates());
        }

        public get ProducedAt(): Date {
            return new Date(this.handle.ProducedAt());
        }

        public get RespNumber(): number {
            return this.handle.RespNumber();
        }

        public RespIndexByCert(cert: pki.Certificate, issuer?: pki.Certificate): number {
            return this.handle.RespIndexByCert(cert.handle, (issuer !== undefined) ? issuer.handle : undefined);
        }

        public get OcspCert(): pki.Certificate | undefined {
            const cert = this.handle.OcspCert();

            if (cert) {
                return pki.Certificate.wrap<native.PKI.Certificate, pki.Certificate>(cert);
            } else {
                return undefined;
            }

        }

        /**
         * Returns OCSP service certificate. if paraneter certs specified then searched through certificates in collection.
         *
         * @param {CertificateCollection} certs
         * @returns {Certificate}
         *
         * @memberOf OCSP
         */
        public getOcspCert(certs?: pki.CertificateCollection): pki.Certificate {
            var cert = undefined;
            if(certs instanceof pki.CertificateCollection) {
                cert = this.handle.getOcspCert(certs.handle);
            }
            else {
                cert = this.handle.getOcspCert();
            }

            if (cert) {
                return pki.Certificate.wrap<native.PKI.Certificate, pki.Certificate>(cert);
            } else {
                return undefined;
            }

        }

        public Status(respIdx?: number): CPCertStatus {
            if(respIdx === undefined)
                return this.handle.Status();

            return this.handle.Status(respIdx);
        }

        RevTime(respIdx?: number): Date {
            if(respIdx === undefined)
                return new Date(this.handle.RevTime());

            return new Date(this.handle.RevTime(respIdx));
        }

        RevReason(respIdx?: number): CPCrlReason {
            if(respIdx === undefined)
                return this.handle.RevReason();

            return this.handle.RevReason(respIdx);
        }

        ThisUpdate(respIdx?: number): Date {
            if(respIdx === undefined)
                return new Date(this.handle.ThisUpdate());

            return new Date(this.handle.ThisUpdate(respIdx));
        }

        /**
         * Return date of Next Update. Field is optional. To verify returned date value call getTime() method on it. If getTime returns 0 than Nextupdate property is empty and should not be used.
         *
         * @readonly
         * @param {number} [respIdx] Response index. Default value is 0.
         * @type {Date}
         * @memberOf OCSP
         */
        NextUpdate(respIdx?: number): Date {
            var nextUpdate;
            if(respIdx === undefined) {
                nextUpdate = this.handle.NextUpdate();
            }
            else {
                nextUpdate = this.handle.NextUpdate(respIdx);
            }

            if("" === nextUpdate) {
                return new Date(0);
            }

            return new Date(nextUpdate);
        }
    }
}
